import 'lodash.defaultsdeep';
import 'crypto';
import 'events';
import 'child_process';
import SQLite from 'better-sqlite3';
import { performance } from 'perf_hooks';
import { ConnectionString } from 'connection-string';
import Debug from 'debug';
import { resolve } from 'path';

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = o[Symbol.iterator]();
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _construct(Parent, args, Class) {
  if (_isNativeReflectConstruct()) {
    _construct = Reflect.construct;
  } else {
    _construct = function _construct(Parent, args, Class) {
      var a = [null];
      a.push.apply(a, args);
      var Constructor = Function.bind.apply(Parent, a);
      var instance = new Constructor();
      if (Class) _setPrototypeOf(instance, Class.prototype);
      return instance;
    };
  }

  return _construct.apply(null, arguments);
}

function _isNativeFunction(fn) {
  return Function.toString.call(fn).indexOf("[native code]") !== -1;
}

function _wrapNativeSuper(Class) {
  var _cache = typeof Map === "function" ? new Map() : undefined;

  _wrapNativeSuper = function _wrapNativeSuper(Class) {
    if (Class === null || !_isNativeFunction(Class)) return Class;

    if (typeof Class !== "function") {
      throw new TypeError("Super expression must either be null or a function");
    }

    if (typeof _cache !== "undefined") {
      if (_cache.has(Class)) return _cache.get(Class);

      _cache.set(Class, Wrapper);
    }

    function Wrapper() {
      return _construct(Class, arguments, _getPrototypeOf(this).constructor);
    }

    Wrapper.prototype = Object.create(Class.prototype, {
      constructor: {
        value: Wrapper,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    return _setPrototypeOf(Wrapper, Class);
  };

  return _wrapNativeSuper(Class);
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (_typeof(call) === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _createSuper(Derived) {
  var hasNativeReflectConstruct = _isNativeReflectConstruct();

  return function _createSuperInternal() {
    var Super = _getPrototypeOf(Derived),
        result;

    if (hasNativeReflectConstruct) {
      var NewTarget = _getPrototypeOf(this).constructor;

      result = Reflect.construct(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }

    return _possibleConstructorReturn(this, result);
  };
}
/* eslint-disable no-dupe-class-members */


var SynorError = /*#__PURE__*/function (_Error) {
  _inherits(SynorError, _Error);

  var _super = _createSuper(SynorError);

  function SynorError(message) {
    var _this;

    var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'exception';
    var data = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

    _classCallCheck(this, SynorError);

    _this = _super.call(this, message);
    Error.captureStackTrace(_assertThisInitialized(_this), _this.constructor);

    if (data instanceof Error) {
      _this.stack = "".concat(data.stack, "\n").concat(_this.stack);
    }

    _this.name = _this.constructor.name;
    _this.type = type;
    _this.data = data;
    return _this;
  }

  return SynorError;
}( /*#__PURE__*/_wrapNativeSuper(Error));

var _iteratorSymbol = /*#__PURE__*/typeof Symbol !== "undefined" ? Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator")) : "@@iterator";

var _iteratorSymbol$1 = /*#__PURE__*/typeof Symbol !== "undefined" ? Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator")) : "@@iterator";
/**
 * JavaScript CRC32 - Stack Overflow - Alex
 * https://stackoverflow.com/a/18639999/4456924
 */


function generateCRC32Table() {
  var crcTable = [];
  var c;

  for (var n = 0; n < 256; n++) {
    c = n;

    for (var k = 0; k < 8; k++) {
      c = c & 1 ? 0xedb88320 ^ c >>> 1 : c >>> 1;
    }

    crcTable[n] = c;
  }

  return crcTable;
}

var crcTable = generateCRC32Table();
/**
 * Generates 32bit signed integer
 */

function crc32(str) {
  var crc = 0 ^ -1;

  for (var i = 0; i < str.length; i++) {
    crc = crc >>> 8 ^ crcTable[(crc ^ str.charCodeAt(i)) & 0xff];
  }

  return (crc ^ -1) >> 0;
}

var synorChecksum = crc32('SYNOR');

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function getQueryStore(database, _ref) {
  var tableName = _ref.migrationRecordTable,
      schemaName = _ref.schemaName;

  var getMigrationTableColumnNames = _async(function () {
    return database.prepare("\n          SELECT info.name AS column_name\n          FROM ".concat(schemaName, ".sqlite_master meta,\n              PRAGMA_TABLE_INFO(meta.tbl_name) info\n          WHERE meta.type = 'table'\n            AND meta.tbl_name = :tableName;\n        ")).all({
      tableName: tableName
    }).map(function (_ref2) {
      var column_name = _ref2.column_name;
      return column_name;
    });
  });

  var createMigrationTable = _async(function () {
    return database.prepare("\n          CREATE TABLE ".concat(schemaName, ".").concat(tableName, " (\n            id INTEGER PRIMARY KEY AUTOINCREMENT,\n            version VARCHAR(128) NOT NULL,\n            type VARCHAR(16),\n            title TEXT,\n            hash TEXT,\n            applied_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,\n            applied_by VARCHAR(255),\n            execution_time INT,\n            dirty TINYINT DEFAULT 0\n          );\n        ")).run();
  }); // const addColumn = {}


  var getLock = _async(function () {
    database.pragma("".concat(schemaName, ".locking_mode=EXCLUSIVE;"));
    return _await();
  });

  var releaseLock = _async(function () {
    database.pragma("".concat(schemaName, ".locking_mode=NORMAL;"));
    return _await();
  });

  var getTableNames = _async(function () {
    return database.prepare("\n          SELECT tbl_name AS table_name\n          FROM ".concat(schemaName, ".sqlite_master\n          WHERE type = 'table';\n        ")).all().map(function (_ref3) {
      var table_name = _ref3.table_name;
      return table_name;
    });
  });

  var dropTables = _async(function (tableNames) {
    var internalSchemaPattern = /^sqlite_/;
    var queries = tableNames.filter(function (tableName) {
      return !internalSchemaPattern.test(tableName);
    }).map(function (tableName) {
      return "DROP TABLE IF EXISTS ".concat(schemaName, ".").concat(tableName, ";");
    }).concat('VACUUM;');

    var _iterator = _createForOfIteratorHelper(queries),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var query = _step.value;
        database.prepare(query).run();
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    return _await();
  });

  var getRecords = _async(function () {
    var startId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    return database.prepare("\n          SELECT\n            id, version, type, title, hash,\n            applied_at AS appliedAt,\n            applied_by AS appliedBy,\n            execution_time AS executionTime,\n            dirty\n          FROM ".concat(schemaName, ".").concat(tableName, "\n          WHERE id >= :startId;\n        ")).all({
      startId: startId
    }).map(function (row) {
      return Object.assign({}, row, {
        dirty: Boolean(row.dirty)
      });
    });
  });

  var addRecord = _async(function (_ref4) {
    var version = _ref4.version,
        type = _ref4.type,
        title = _ref4.title,
        hash = _ref4.hash,
        appliedAt = _ref4.appliedAt,
        appliedBy = _ref4.appliedBy,
        executionTime = _ref4.executionTime,
        dirty = _ref4.dirty;
    return database.prepare("\n          INSERT INTO ".concat(schemaName, ".").concat(tableName, " (\n            version, type, title, hash, applied_at, applied_by, execution_time, dirty\n          ) VALUES (\n            :version, :type, :title, :hash, :appliedAt, :appliedBy, :executionTime, :dirty\n          );\n        ")).run({
      version: version,
      type: type,
      title: title,
      hash: hash,
      appliedAt: appliedAt.toISOString(),
      appliedBy: appliedBy,
      executionTime: executionTime,
      dirty: +dirty
    });
  });

  var deleteDirtyRecords = _async(function () {
    return database.prepare("DELETE FROM ".concat(schemaName, ".").concat(tableName, " WHERE dirty = :dirty;")).run({
      dirty: 1
    });
  });

  var updateRecord = _async(function (id, _ref5) {
    var hash = _ref5.hash;
    return database.prepare("\n          UPDATE ".concat(schemaName, ".").concat(tableName, "\n            SET hash = :hash\n            WHERE id = :id;\n        ")).run({
      hash: hash,
      id: id
    });
  });

  return {
    getMigrationTableColumnNames: getMigrationTableColumnNames,
    createMigrationTable: createMigrationTable,
    // addColumn,
    getLock: getLock,
    releaseLock: releaseLock,
    getTableNames: getTableNames,
    dropTables: dropTables,
    getRecords: getRecords,
    addRecord: addRecord,
    deleteDirtyRecords: deleteDirtyRecords,
    updateRecord: updateRecord
  };
}

function _empty() {}

function _awaitIgnored(value, direct) {
  if (!direct) {
    return value && value.then ? value.then(_empty) : Promise.resolve();
  }
}

function _invokeIgnored(body) {
  var result = body();

  if (result && result.then) {
    return result.then(_empty);
  }
}

function _invoke(body, then) {
  var result = body();

  if (result && result.then) {
    return result.then(then);
  }

  return then(result);
}

function _await$1(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function _async$1(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

var ensureMigrationRecordTable = _async$1(function (queryStore, baseVersion) {
  return _await$1(queryStore.getMigrationTableColumnNames(), function (existingColumnNames) {
    var migrationTableExists = Boolean(existingColumnNames.length);
    var startTime = performance.now();
    return _invoke(function () {
      if (!migrationTableExists) {
        return _awaitIgnored(queryStore.createMigrationTable());
      }
    }, function () {
      // for (const [columnName, addColumn] of Object.entries(queryStore.addColumn)) {
      //   if (!existingColumnNames.includes(columnName)) {
      //     await addColumn!()
      //   }
      // }
      var endTime = performance.now();
      return _invokeIgnored(function () {
        if (!migrationTableExists) {
          return _awaitIgnored(queryStore.addRecord({
            version: baseVersion,
            type: 'do',
            title: 'Base Migration',
            hash: '',
            appliedAt: new Date(),
            appliedBy: 'Synor',
            executionTime: endTime - startTime,
            dirty: false
          }));
        }
      });
    });
  });
});

var debug = Debug('@synor/database-sqlite');
var validHost = ['', '.'];
function getConfig(uri) {
  try {
    var _ConnectionString = new ConnectionString(uri, {
      params: {
        schema: 'main',
        synor_migration_record_table: 'synor_migration_record',
        file_must_exist: false,
        memory: false,
        readonly: false,
        timeout: 5000
      }
    }),
        protocol = _ConnectionString.protocol,
        _ConnectionString$hos = _ConnectionString.hostname,
        host = _ConnectionString$hos === void 0 ? '' : _ConnectionString$hos,
        path = _ConnectionString.path,
        params = _ConnectionString.params;

    if (!protocol) {
      throw new Error("[URI] missing: protocol!");
    }

    if (protocol !== 'sqlite') {
      throw new Error("[URI] unsupported: protocol(".concat(protocol, ")!"));
    }

    if (!validHost.includes(host)) {
      throw new Error("[URI] unsupported: host(".concat(host, ")!"));
    }

    var filepath;

    if (!path) {
      debug('[URI] missing: path! Using in-memory database...');
      params.memory = true;
    }

    if (params.memory) {
      filepath = ':memory:';
    } else {
      filepath = resolve("".concat(host, "/").concat(path.join('/')));
    }

    var databaseConfig = {
      filepath: filepath,
      fileMustExist: JSON.parse(params.file_must_exist),
      memory: JSON.parse(params.memory),
      readonly: JSON.parse(params.readonly),
      timeout: JSON.parse(params.timeout)
    };
    var engineConfig = {
      migrationRecordTable: params.synor_migration_record_table,
      schema: params.schema
    };
    return {
      databaseConfig: databaseConfig,
      engineConfig: engineConfig
    };
  } catch (error) {
    throw new SynorError('Invalid DatabaseURI', 'exception', error);
  }
}

function _empty$1() {}

function _awaitIgnored$1(value, direct) {
  if (!direct) {
    return value && value.then ? value.then(_empty$1) : Promise.resolve();
  }
}

function _call(body, then, direct) {
  if (direct) {
    return then ? then(body()) : body();
  }

  try {
    var result = Promise.resolve(body());
    return then ? result.then(then) : result;
  } catch (e) {
    return Promise.reject(e);
  }
}

function _await$2(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function _async$2(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function _invokeIgnored$1(body) {
  var result = body();

  if (result && result.then) {
    return result.then(_empty$1);
  }
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function _rethrow(thrown, value) {
  if (thrown) throw value;
  return value;
}

function _finallyRethrows(body, finalizer) {
  try {
    var result = body();
  } catch (e) {
    return finalizer(true, e);
  }

  if (result && result.then) {
    return result.then(finalizer.bind(null, false), finalizer.bind(null, true));
  }

  return finalizer(false, result);
}

var _iteratorSymbol$2 = /*#__PURE__*/typeof Symbol !== "undefined" ? Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator")) : "@@iterator";

function _settle(pact, state, value) {
  if (!pact.s) {
    if (value instanceof _Pact) {
      if (value.s) {
        if (state & 1) {
          state = value.s;
        }

        value = value.v;
      } else {
        value.o = _settle.bind(null, pact, state);
        return;
      }
    }

    if (value && value.then) {
      value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
      return;
    }

    pact.s = state;
    pact.v = value;
    var observer = pact.o;

    if (observer) {
      observer(pact);
    }
  }
}

var _Pact = /*#__PURE__*/function () {
  function _Pact() {}

  _Pact.prototype.then = function (onFulfilled, onRejected) {
    var result = new _Pact();
    var state = this.s;

    if (state) {
      var callback = state & 1 ? onFulfilled : onRejected;

      if (callback) {
        try {
          _settle(result, 1, callback(this.v));
        } catch (e) {
          _settle(result, 2, e);
        }

        return result;
      } else {
        return this;
      }
    }

    this.o = function (_this) {
      try {
        var value = _this.v;

        if (_this.s & 1) {
          _settle(result, 1, onFulfilled ? onFulfilled(value) : value);
        } else if (onRejected) {
          _settle(result, 1, onRejected(value));
        } else {
          _settle(result, 2, value);
        }
      } catch (e) {
        _settle(result, 2, e);
      }
    };

    return result;
  };

  return _Pact;
}();

function _isSettledPact(thenable) {
  return thenable instanceof _Pact && thenable.s & 1;
}

function _forTo(array, body, check) {
  var i = -1,
      pact,
      reject;

  function _cycle(result) {
    try {
      while (++i < array.length && (!check || !check())) {
        result = body(i);

        if (result && result.then) {
          if (_isSettledPact(result)) {
            result = result.v;
          } else {
            result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
            return;
          }
        }
      }

      if (pact) {
        _settle(pact, 1, result);
      } else {
        pact = result;
      }
    } catch (e) {
      _settle(pact || (pact = new _Pact()), 2, e);
    }
  }

  _cycle();

  return pact;
}

function _forOf(target, body, check) {
  if (typeof target[_iteratorSymbol$2] === "function") {
    var _cycle2 = function _cycle2(result) {
      try {
        while (!(step = iterator.next()).done && (!check || !check())) {
          result = body(step.value);

          if (result && result.then) {
            if (_isSettledPact(result)) {
              result = result.v;
            } else {
              result.then(_cycle2, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
              return;
            }
          }
        }

        if (pact) {
          _settle(pact, 1, result);
        } else {
          pact = result;
        }
      } catch (e) {
        _settle(pact || (pact = new _Pact()), 2, e);
      }
    };

    var iterator = target[_iteratorSymbol$2](),
        step,
        pact,
        reject;

    _cycle2();

    if (iterator["return"]) {
      var _fixup = function _fixup(value) {
        try {
          if (!step.done) {
            iterator["return"]();
          }
        } catch (e) {}

        return value;
      };

      if (pact && pact.then) {
        return pact.then(_fixup, function (e) {
          throw _fixup(e);
        });
      }

      _fixup();
    }

    return pact;
  } // No support for Symbol.iterator


  if (!("length" in target)) {
    throw new TypeError("Object is not iterable");
  } // Handle live collections properly


  var values = [];

  for (var i = 0; i < target.length; i++) {
    values.push(target[i]);
  }

  return _forTo(values, function (i) {
    return body(values[i]);
  }, check);
}

function _continueIgnored(value) {
  if (value && value.then) {
    return value.then(_empty$1);
  }
}

var SQLiteDatabaseEngine = function SQLiteDatabaseEngine(uri, _ref) {
  var baseVersion = _ref.baseVersion,
      getUserInfo = _ref.getUserInfo;

  var _getConfig = getConfig(uri),
      databaseConfig = _getConfig.databaseConfig,
      engineConfig = _getConfig.engineConfig;

  if (typeof getUserInfo !== 'function') {
    throw new SynorError("Missing: getUserInfo");
  }

  var database = SQLite(databaseConfig.filepath, {
    fileMustExist: databaseConfig.fileMustExist,
    memory: databaseConfig.memory,
    readonly: databaseConfig.readonly,
    timeout: databaseConfig.timeout
  });
  var queryStore = getQueryStore(database, {
    migrationRecordTable: engineConfig.migrationRecordTable,
    schemaName: engineConfig.schema
  });
  var appliedBy = '';

  var open = function open() {
    return _call(getUserInfo, function (_getUserInfo) {
      appliedBy = _getUserInfo;
      return _awaitIgnored$1(ensureMigrationRecordTable(queryStore, baseVersion));
    });
  };

  var close = _async$2(function () {
    database.close();
    return _await$2();
  });

  var lock = _async$2(function () {
    return _awaitIgnored$1(queryStore.getLock());
  });

  var unlock = _async$2(function () {
    return _awaitIgnored$1(queryStore.releaseLock());
  });

  var drop = _async$2(function () {
    return _await$2(queryStore.getTableNames(), function (tableNames) {
      return _awaitIgnored$1(queryStore.dropTables(tableNames));
    });
  });

  var run = _async$2(function (_ref2) {
    var version = _ref2.version,
        type = _ref2.type,
        title = _ref2.title,
        hash = _ref2.hash,
        body = _ref2.body,
        run = _ref2.run;
    var dirty = false;
    var startTime = performance.now();
    return _finallyRethrows(function () {
      return _catch(function () {
        return _invokeIgnored$1(function () {
          if (body) {
            database.exec(body);
          } else {
            return _awaitIgnored$1(run(database));
          }
        });
      }, function (err) {
        dirty = true;
        throw err;
      });
    }, function (_wasThrown, _result) {
      var endTime = performance.now();
      return _await$2(queryStore.addRecord({
        version: version,
        type: type,
        title: title,
        hash: hash,
        appliedAt: new Date(),
        appliedBy: appliedBy,
        executionTime: endTime - startTime,
        dirty: dirty
      }), function () {
        return _rethrow(_wasThrown, _result);
      });
    });
  });

  var repair = _async$2(function (records) {
    return _await$2(queryStore.deleteDirtyRecords(), function () {
      return _continueIgnored(_forOf(records, function (_ref3) {
        var id = _ref3.id,
            hash = _ref3.hash;
        return _awaitIgnored$1(queryStore.updateRecord(id, {
          hash: hash
        }));
      }));
    });
  });

  var records = _async$2(function (startId) {
    return queryStore.getRecords(startId);
  });

  return {
    open: open,
    close: close,
    lock: lock,
    unlock: unlock,
    drop: drop,
    run: run,
    repair: repair,
    records: records
  };
};

export default SQLiteDatabaseEngine;
export { SQLiteDatabaseEngine };
